/********************************************
*       ORBITER MODULE: Mission Timer
*     Made for Project Apollo Missions
*            by Dave Gundlach
*               Jan 2010
*
* Update 2/13/10 - Fixed negative number entry
* Update 4/11/10 - Modified the Acceleration
* entry and added input box
*********************************************/

//#define STRICT
#define ORBITER_MODULE
// To force orbitersdk.h to use <fstream> in any compiler version
#pragma include_alias( <fstream.h>, <fstream> )
#include "windows.h"
#include <stdio.h>
#include "math.h"
#include "orbitersdk.h"
#include "nasspsound.h"
#include "soundlib.h"
#include "saturn.h"
#include "LEM.h"
#include "Crawler.h"
#include "MFDResource.h"
#include <queue>
#include "MtimerMFD.h"

// ==============================================================
// Global variables
int mode;			// identifier for new MFD mode
bool ON = false;	// Timer Status On/Off
bool OFF = false;	// Timer Off flag
int TP_H = 0;		// Time to Pause Hours
int TP_M = 0;		// Time to Pause Minutes
int TTP = 0;		// Time to Pause
double AT = 10;		// Acceleration Factor
int MH = 0;				// MET Hours
int MM = 0;				// MET Minutes
int MS = 0;				// MET Seconds
double MaxA = 60;		// Max Acceleration Value
double MinA = 1;		// Minimum Acceleration Value
// global data storage (from Project Apollo MFDs)
static struct MtimerMFDData {  
		Saturn *progVessel;
		LEM *gorpVessel;
}g_Data;
// ==============================================================
// API interface

DLLCLBK void opcDLLInit (HINSTANCE hDLL)
{
	static char *name = "Mission Pause";
	MFDMODESPEC spec;
	spec.name	 = name;g_Data;
	spec.key	 = OAPI_KEY_P;
	spec.msgproc = MtimerMFD::MsgProc;

	mode		 = oapiRegisterMFDMode (spec);
}

DLLCLBK void opcDLLExit (HINSTANCE hDLL)
{
	oapiUnregisterMFDMode (mode);	
}

DLLCLBK void opcTimestep (double simt, double simdt, double mjd)
{
	
}

// ==============================================================
// MFD implementation

MtimerMFD::MtimerMFD (DWORD w, DWORD h, VESSEL *vessel)
: MFD (w, h, vessel)
{
	saturn = NULL;
	crawler = NULL;
	lem = NULL;
	
	width = w;
	height = h;

	//We need to find out what type of vessel it is, so we check for the class name.
	//Saturns have different functions than Crawlers.  But we have methods for both.
	//(From Project Apollo MFDs)
	if (!stricmp(vessel->GetClassName(), "ProjectApollo\\Saturn5") ||
		!stricmp(vessel->GetClassName(), "ProjectApollo/Saturn5") ||
		!stricmp(vessel->GetClassName(), "ProjectApollo\\Saturn1b") ||
		!stricmp(vessel->GetClassName(), "ProjectApollo/Saturn1b")) {
		saturn = (Saturn *)vessel;
	}
	else if (!stricmp(vessel->GetClassName(), "ProjectApollo\\Crawler") ||
		!stricmp(vessel->GetClassName(), "ProjectApollo/Crawler"))  {
			crawler = (Crawler *)vessel;
	}
	else if (!stricmp(vessel->GetClassName(), "ProjectApollo\\LEM") ||
		!stricmp(vessel->GetClassName(), "ProjectApollo/LEM")) {
			lem = (LEM *)vessel;
	}	
}

MtimerMFD::~MtimerMFD ()
{
	
}
// message parser
int MtimerMFD::MsgProc (UINT msg, UINT mfd, WPARAM wparam, LPARAM lparam)
{
	switch (msg) {
	case OAPI_MSG_MFD_OPENED:
		return (int)(new MtimerMFD (LOWORD(wparam), HIWORD(wparam), (VESSEL*)lparam));
	}
	return 0;
}

bool MtimerMFD::ConsumeKeyBuffered (DWORD key)
{
	// Actions when keypress is detected

	bool METInput (void *id, char *str, void *data);	// MET Input Box
	bool TAInput (void *id, char *str, void *data);	// Time Accel Input Box

	switch (key) {
	case OAPI_KEY_B:	//MET 
		oapiOpenInputBox ("MET [HH MM SS]:", METInput, 0, 20, (void*)this);
		return true;
	case OAPI_KEY_I:	// On
		ON = true;
		return true;
	case OAPI_KEY_O:	// OFF
		ON = false;
		OFF = true;
		return true;	// H+
	case OAPI_KEY_H:
		TTP = TTP + 3600;
		return true;
	case OAPI_KEY_G:	// H-
		TTP = TTP - 3600;
		return true;
	case OAPI_KEY_M:	// M+
		TTP = TTP + 60;
		return true;
	case OAPI_KEY_N:	// M-
		TTP = TTP - 60;
		return true;
	case OAPI_KEY_S:	// S+
		TTP = TTP + 1;
		return true;
	case OAPI_KEY_A:	// S-
		TTP = TTP - 1;
		return true;
	case OAPI_KEY_T:	// A+
		AT = AT + 1;
		if (AT > MaxA)
			AT = MaxA;		// Limit max to MaxA defined above
		return true;
	case OAPI_KEY_R:	// A-
		AT = AT - 1;
		if (AT < MinA)
			AT = MinA;		// Limit min Acceleration to MinA defined above
		return true;
	case OAPI_KEY_W:	// Time Acceleration Input Box
		oapiOpenInputBox ("Time Acceleration:", TAInput, 0, 20, (void*)this);
		return true;
	}
	return false;
}

bool MtimerMFD::ConsumeButton (int bt, int event)
{
		// Keypress when MFD button is pressed

	if (!(event & PANEL_MOUSE_LBDOWN)) return false;
	static const DWORD btkey[12] = {OAPI_KEY_T, OAPI_KEY_R, OAPI_KEY_B, OAPI_KEY_W, OAPI_KEY_I, OAPI_KEY_O, OAPI_KEY_H, OAPI_KEY_G, OAPI_KEY_M, OAPI_KEY_N, OAPI_KEY_S, OAPI_KEY_A };
	if (bt <12) return ConsumeKeyBuffered (btkey[bt]);
	else return false;
}

char *MtimerMFD::ButtonLabel (int bt)
{
		// Button Labels

	char *label[12] = {"A+", "A-", "MET", "TA", "ON", "OFF", "H+", "H-","M+","M-","S+", "S-"};
	return (bt < 12 ? label[bt] : 0);
}

int MtimerMFD::ButtonMenu (const MFDBUTTONMENU **menu) const
{
	// MFD Button Menu

	static const MFDBUTTONMENU mnu[12] = {
		{"Accelerate Time +", 0, 'T'},
		{"Accelerate Time -", 0, 'R'},
		{"Input MET", 0, 'B'},
		{"Input Time Acceleration", 0,'W'},
		{"Turn On Timer", 0, 'I'},
		{"Turn Off Timer", 0, 'O'},
		{"MET Hours +", 0, 'H'},
		{"MET Hours -", 0, 'G'},
		{"MET Minutes +", 0, 'M'},
		{"MET Minutes -", 0, 'N'}, 
		{"MET Seconds +", 0, 'S'},
		{"MET Seconds -", 0, 'A'},
	};
	if (menu) *menu = mnu;
	return 12;
}

void MtimerMFD::Update (HDC hDC)
{
	Title (hDC, "Mission Pause");

	char buffer[100];
	double sim_accel;						// Sim Acceleration

	sim_accel = oapiGetTimeAcceleration ();	// get sim time acceleration
	
	SelectDefaultFont(hDC, 0);				// From Project Apollo MFDs
	SetBkMode (hDC, TRANSPARENT);
	SetTextAlign (hDC, TA_CENTER);

	if (!saturn && !lem)					// Check for supported vessel
	{	SetTextColor (hDC, RGB(255, 0, 0));
		TextOut(hDC, width / 2, (int) (height * 0.5), "Unsupported vessel", 18); 
		if (!crawler)
			return;
	}

	// Draw mission time - From Project Apollo MFD's

	SetTextColor (hDC, RGB(0, 255, 0));
	TextOut(hDC, width / 2, (int) (height * 0.1), "Ground Elapsed Time", 19);

	double mt = 0;
	if (saturn){ mt = saturn->GetMissionTime(); }
	if (crawler){ mt = crawler->GetMissionTime(); }
	if (lem){ mt = lem->GetMissionTime(); }

	int secs = abs((int) mt);
	int hours = (secs / 3600);
	secs -= (hours * 3600);
	int minutes = (secs / 60);
	secs -= 60 * minutes;
	if (mt < 0)
		sprintf(buffer, "-%d:%02d:%02d", hours, minutes, secs);
	else
		sprintf(buffer, "%d:%02d:%02d", hours, minutes, secs);
	TextOut(hDC, width / 2, (int) (height * 0.15), buffer, strlen(buffer));

	//If this is the crawler and not the actual Saturn, do NOTHING else!

	if (!saturn && !lem)
	return;

	SelectDefaultPen(hDC, 1);
	MoveToEx (hDC, (int) (width * 0.05), (int) (height * 0.25), 0);
	LineTo (hDC, (int) (width * 0.95), (int) (height * 0.25));

	if (ON == false)						 // Timer is OFF
	{	
		if (OFF == true)					// check if Off was pressed after timer start
		{
			oapiSetTimeAcceleration (1.0);	// Set Acceleration to 1
			OFF = false;
		}
		SetTextAlign (hDC, TA_LEFT);
		SetTextColor (hDC, RGB(120, 120, 120));	//text color grey
		sprintf(buffer, "Current Time Accel     : %.2f ", sim_accel ); 
		TextOut(hDC, (int)(width * 0.01), (int) (height * 0.25), buffer,strlen(buffer));
		SetTextColor (hDC, RGB(0, 255, 0));		//text color green
		sprintf(buffer, "Set Time Accel         : %.2f ", AT ); 
		TextOut(hDC, (int)(width * 0.01), (int) (height * 0.3), buffer,strlen(buffer));
		
		MS = abs ((int) TTP);
		MH = (MS / 3600);
		MS -= (MH * 3600);
		MM = (MS / 60);
		MS -= 60 * MM;

		if (TTP < 0)
			sprintf(buffer, "Set MET for next Pause : -%i:%02i:%02i", MH, MM, MS);
		else	
			sprintf(buffer, "Set MET for next Pause : %i:%02i:%02i ", MH, MM, MS );

		TextOut(hDC, (int)(width * 0.01), (int) (height * 0.4),  buffer,strlen(buffer));
		TextOut(hDC, (int)(width * 0.01), (int) (height * 0.9), "Timer Status: OFF",17);
		
	}
	
	else										// Timer is ON
	{	if ((mt + 0.5) <= TTP) 
		{	oapiSetTimeAcceleration (AT);
			SetTextAlign (hDC, TA_LEFT);
			SetTextColor (hDC, RGB(120, 120, 120));	//text color grey
			sprintf(buffer, "Current Time Accel     : %.2f ", sim_accel ); 
			TextOut(hDC, (int)(width * 0.01), (int) (height * 0.25), buffer,strlen(buffer));
			SetTextColor (hDC, RGB(0, 255, 0));		//text color green
			MS = abs ((int) TTP);
			MH = (MS / 3600);
			MS -= (MH * 3600);
			MM = (MS / 60);
			MS -= 60 * MM;
			if (TTP < 0)
				sprintf(buffer, "    MET for next Pause : -%i:%02i:%02i", MH, MM, MS);
			else	
				sprintf(buffer, "    MET for next Pause : %i:%02i:%02i ", MH, MM, MS );

			TextOut(hDC, (int)(width * 0.01), (int) (height * 0.4),  buffer,strlen(buffer));
			TextOut(hDC, (int)(width * 0.01), (int) (height * 0.9), "Timer Status: ON",16);
		}

		else									// Timer Triggered
		{	SetTextColor (hDC, RGB(255, 0, 0));	// Text color red
			TextOut(hDC, (int)(width / 2), (int) (height * 0.9), "Timer Status: PAUSE",19);
			oapiSetTimeAcceleration (1.0);		// Set Acceleration to 1
			ON = false;							// Turn Timer off
			oapiSetPause (true);				// Pause simulation
		}
	}
}
bool MtimerMFD::SetMET (char *rstr)			// MET Input Box
{
	char SN;	// Sign
	int H;		// Hour
	int M;		// Minute
	int S;		// Second
	sscanf (rstr, "%c%d%d", &SN, &M, &S);
		if (SN == '-')						//check for a negative time
		{
			sscanf (rstr, "%d%d%d", &H, &M, &S);
			H = -abs(H);
			M = -M;
			S = -S;
		}
		else
		{
			sscanf (rstr, "%d%d%d", &H, &M, &S);
		}
		TTP = (H * 3600) + (M * 60) + S;
		return true;
}
bool MtimerMFD::SetTA (char *rstr)			// Time Acceleration Input Box
{	
	float warp;
	sscanf (rstr, "%f", &warp);
	if (warp < MinA)
	{
		warp = MinA;
	}
	if (warp > MaxA)
	{
			warp = MaxA;
	}
	AT = warp;
	return true;
}
bool METInput (void *id, char *str, void *data)	
{
	return ((MtimerMFD*)data)->SetMET (str);
}
bool TAInput (void *id, char *str, void *data)	
{
	return ((MtimerMFD*)data)->SetTA (str);
}